<?php

require 'classes/Quest.php';

class CommanderController{

    protected function getMysql($account_id)
    {
        $mysql_conf = getMysqlConfig(crc32($account_id));
        $conn = new phpcommon\Mysql(array(
            'host' => $mysql_conf['host'],
            'port' => $mysql_conf['port'],
            'user' => $mysql_conf['user'],
            'passwd' => $mysql_conf['passwd'],
            'dbname' => 'gamedb2002_' . $mysql_conf['instance_id']
        ));
        return $conn;
    }

    protected function getCommander($commander_id)
    {
        $commander_meta_table = require('../res/driver@driver.php');
        $commander_meta = getCommanderConfig($commander_meta_table, $commander_id);
        $c = array(
            'id' => $commander_meta['id'],
            'name' => $commander_meta['name'],
            'compose' => $commander_meta['compose'],
            'debris_id' => $commander_meta['debris'],
            'g_type' => $commander_meta['g_type'],
            'cost' => $commander_meta['cost'],
            'cost_int' => $commander_meta['cost_int'],
            'gold_cost_int' => $commander_meta['gold_cost_int'],
            'gold_cost' => $commander_meta['gold_cost'],
            'max_lv' => $commander_meta['max_lv'],
        );
        return $c;
    }

    protected function getExplode($string)
    {
        $delim = "|";
        $drop_multiply = explode($delim, $string);
        $delim1 = ":";
        $arr = array();
        for ($i = 0; $i < count($drop_multiply); $i++) {
            $mul = explode($delim1, $drop_multiply[$i]);
            array_push($arr, $mul);
        }
        return $arr;
    }

    protected function getParameter($para_id)
    {
        $parameter_meta_cluster = require('../res/parameter@parameter.php');
        $parameter_meta = getParameterConfig($parameter_meta_cluster, $para_id);
        $p = array(
            'id' => $parameter_meta['id'],
            'param_name' => $parameter_meta['param_name'],
            'param_value' => $parameter_meta['param_value'],
        );
        return $p;
    }

    protected function updateActiveTime($row, $conn)
    {
        $ret = $conn->execScript('UPDATE commander SET active_time=0,experience_level=0, modify_time=:modify_time ' .
                                 ' WHERE accountid=:account_id AND commander_id=:commander_id;',
                                 array(
                                     ':account_id' => $row['accountid'],
                                     ':commander_id' => $row['commander_id'],
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
    }

    public function commanderInfo()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        $commander_list = array();
        if(!$conn){
            phpcommon\sendError(ERR_USER_BASE + 1,'没有这个玩家');
            return;
        }
        $commander_meta_table = require('../res/driver@driver.php');
        $c = $this->getCommander(14001);
        if (!$c) {
            phpcommon\sendError(ERR_USER_BASE + 1,'没有这个指挥官');
            return;
        }
        $rowCount = $conn->execQueryRowCount('SELECT commander_id FROM commander WHERE accountid = :account_id;',
                                             array(
                                                 ':account_id' => $account_id
                                             ));
        $commander_status = 0;
        if ($rowCount == 0) {
            for ($i = 14001; $i <= count($commander_meta_table) + 14000; $i++) {
                $c = $this->getCommander($i);
                if (!$c) {
                    phpcommon\sendError(ERR_USER_BASE + 1,'没有这个指挥官');
                    return;
                }
                if ($i == 14001) {
                    $commander_status = 0;
                } else {
                    $commander_status = 2;
                }
                $ret = $conn->execScript('INSERT INTO commander(accountid, commander_id, commander_status, fragment_id, fragment_num, commander_type, commander_level, experience_level, active_time, create_time, modify_time) ' .
                                         ' VALUES(:account_id, :commander_id, :commander_status, :fragment_id, 100000, :commander_type, 1, 0, 0, :create_time, :modify_time) ' .
                                         ' ON DUPLICATE KEY UPDATE accountid=:account_id, commander_id=:commander_id, commander_status=:commander_status, fragment_id=:fragment_id, fragment_num=100000, commander_type=:commander_type, commander_level=1, experience_level=0, active_time=0, modify_time=:modify_time;',
                                         array(
                                             ':account_id' => $account_id,
                                             ':commander_id' => $i,
                                             ':commander_status' => $commander_status,
                                             ':fragment_id' => $c['debris_id'],
                                             ':commander_type' => $c['g_type'],
                                             ':create_time' => time(),
                                             ':modify_time' => time()
                                         ));
                if(!$ret){
                    die();
                    return;
                }
            }
        }
        $rows = $conn->execQuery('SELECT * FROM commander WHERE accountid = :account_id;',
                                 array(
                                     ':account_id' => $account_id,
                                 ));
        $commander_id = 0;
        $commander_level = 0;
        foreach ($rows as $row) {
            $time = 0;
            $commander_level = $row['commander_level'];
            $c = $this->getCommander($row['commander_id']);
            if ($commander_id == $row['commander_id']) {
                continue;
            }
            if ($row['active_time'] != 0) {
                if(time() >= $row['active_time']){
                    $this->updateActiveTime($row, $conn);
                    $time = 0;
                } else {
                    $commander_level = $row['experience_level'];
                    $time = $row['active_time'];
                }
            }
            $commander_id = $row['commander_id'];
            array_push($commander_list, array(
                'commander_id' => $commander_id,
                'commander_status' => $row['commander_status'],
                'fragment_num' => $row['fragment_num'],
                'commander_level' => $commander_level,
                'time' => $time,
            ));
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'commander_list' => $commander_list
        ));
    }


    public function unlockCommander()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        $commander_id = $_REQUEST['commander_id'];
        if(!$conn){
            phpcommon\sendError(ERR_USER_BASE + 1,'没有这个玩家');
            return;
        }
        $c = $this->getCommander($commander_id);
        if(!$c){
            phpcommon\sendError(ERR_USER_BASE + 2,'没有这个指挥官');
            return;
        }
        $row = $conn->execQueryOne('SELECT * FROM commander WHERE accountid=:account_id AND commander_id=:commander_id;',
                                   array(
                                       ':account_id' => $account_id,
                                       ':commander_id' => $commander_id
                                   ));
        if(!$row){
            phpcommon\sendError(ERR_USER_BASE + 2,'没有这个指挥官');
            return;
        }
        $fragment_num = 0;
        $array = $this->getExplode($c['compose']);
        if ($row['fragment_num'] < $array[0][1]) {
            phpcommon\sendError(ERR_USER_BASE + 4,'指挥官碎片数量不足');
            return;
        }
        $fragment_num = $row['fragment_num'] - $array[0][1];

        $commander_status = 1;
        $ret = $conn->execScript('UPDATE commander SET fragment_num=:fragment_num ,commander_status=:commander_status, modify_time=:modify_time ' .
                                 ' WHERE accountid = :account_id AND commander_id = :commander_id;',
                                 array(
                                     ':account_id' => $account_id,
                                     ':commander_id' => $commander_id,
                                     ':fragment_num' => $fragment_num,
                                     ':commander_status' => $commander_status,
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
            return;
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            "commander_id" => $commander_id,
            "commander_status" => $commander_status,
            "fragment_num" => $fragment_num,
            "commander_level" => 1,
        ));
    }

    public function exchangeCommander()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $usingcommander_id = $_REQUEST['usingcommander_id'];
        $exchangecommander_id = $_REQUEST['exchangecommander_id'];
        $conn = $this->getMysql($account_id);
        if(!$conn){
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        //正在上阵的指挥官
        $row = $conn->execQueryOne('SELECT * FROM commander WHERE accountid=:accountid AND commander_id=:commander_id;',
                                   array(
                                       ':accountid' => $account_id,
                                       ':commander_id' => $usingcommander_id,
                                   ));
        if (!$row) {
            phpcommon\sendError(ERR_USER_BASE + 2, '没有这个指挥官');
            return;
        }
        $commander_status = 1;
        $using_ret = $conn->execScript('UPDATE commander SET commander_status=:commander_status, modify_time=:modify_time ' .
                                       ' WHERE accountid = :account_id AND commander_id = :commander_id;',
                                       array(
                                           ':account_id' => $account_id,
                                           ':commander_id' => $usingcommander_id,
                                           ':commander_status' => $commander_status,
                                           ':modify_time' => time()
                                       ));

        if(!$using_ret){
            die();
            return;
        }
        $commander_list = array();
        array_push($commander_list, array(
            'commander_id' => $usingcommander_id,
            'commander_status' => $commander_status,
        ));
        //准备上阵的指挥官
        $commander_status = 0;
        $row_exchange = $conn->execQueryOne('SELECT * FROM commander WHERE accountid=:accountid AND commander_id=:commander_id;',
                                            array(
                                                 ':accountid' => $account_id,
                                                 ':commander_id' => $exchangecommander_id,
                                            ));
        $exchange_ret = $conn->execScript('UPDATE commander SET commander_status=0, modify_time=:modify_time ' .
                                          ' WHERE accountid = :account_id AND commander_id = :commander_id;',
                                          array(
                                              ':account_id' => $account_id,
                                              ':commander_id' => $exchangecommander_id,
                                              ':modify_time' => time()
                                          ));
        if(!$exchange_ret){
            die();
            return;
        }
        array_push($commander_list, array(
            'commander_id' => $exchangecommander_id,
            'commander_status' => $commander_status,
        ));

        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            "commander_list" => $commander_list,
        ));
    }

    public function updateCommander()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个角色');
            return;
        }
        $commander_id = $_REQUEST['commander_id'];
        $free = $_REQUEST['free'];
        $row = $conn->execQueryOne('SELECT * FROM commander WHERE accountid=:account_id AND commander_id=:commander_id;',
                                   array(
                                       ':account_id' => $account_id,
                                       ':commander_id' => $commander_id
                                   ));
        if (!$row) {
            phpcommon\sendError(ERR_USER_BASE + 1, '指挥官不存在');
            return;
        }
        $rowCoin = $conn->execQueryOne('SELECT coin_num FROM user WHERE accountid=:account_id;',
                                       array(
                                           ':account_id' => $account_id,
                                       ));
        if (!$rowCoin) {
            phpcommon\sendError(ERR_USER_BASE + 1, '指挥官不存在');
            return;
        }
        $c = $this->getCommander($commander_id);
        if (!$c) {
            phpcommon\sendError(ERR_USER_BASE + 3, '没有这个指挥官');
            return;
        }
        if ($row['commander_level'] >= $c['max_lv']) {
            phpcommon\sendError(ERR_USER_BASE + 4, '指挥官已到满级');
            return;
        }
        $debris_num = $c['cost'] * ceil($row['commander_level'] / 5) + $c['cost_int'];
        $coin_num = $c['gold_cost'] * ceil($row['commander_level'] / 5) + $c['gold_cost_int'];
        if ($free == 1) {
            $debris_num = 0;
            $coin_num = 0;
        }
        if ($debris_num > $row['fragment_num']) {
            phpcommon\sendError(ERR_USER_BASE + 5, '指挥官碎片数量不足');
            return;
        }
        if ($coin_num > $rowCoin['coin_num']) {
            phpcommon\sendError(ERR_USER_BASE + 6, '金币不足');
            return;
        }
        $ret = $conn->execScript('UPDATE commander SET commander_level=:commander_level, fragment_num=:fragment_num, modify_time=:modify_time ' .
                                 ' WHERE accountid=:account_id AND commander_id=:commander_id;',
                                 array(
                                     ':account_id' => $account_id,
                                     ':commander_id' => $commander_id,
                                     ':commander_level' => $row['commander_level'] + 1,
                                     ':fragment_num' => $row['fragment_num'] - $debris_num,
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
            return;
        }
        $retCoin = $conn->execScript('UPDATE user SET coin_num=:coin_num, modify_time=:modify_time ' .
                                     ' WHERE accountid=:account_id;',
                                     array(
                                         ':account_id' => $account_id,
                                         ':coin_num' => $rowCoin['coin_num'] - $coin_num,
                                         ':modify_time' => time()
                                     ));
        if (!$retCoin) {
            die();
            return;
        }
        $quest = new classes\Quest();
        $quest->triggerQuest(QUEST_DAY_UPDATEDRIVER, 1, 1, $account_id);
        if ($row['commander_level'] + 1 >= $c['max_lv']) {
            $quest->triggerQuest(QUEST_SUM_DRIVER, 2, 1, $account_id);
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'commander_id' => $row['commander_id'],
            'commander_status' => $row['commander_status'],
            'fragment_num' => $row['fragment_num'] - $debris_num,
            'commander_level' => $row['commander_level'] + 1,
        ));
    }

    public function freetrycommander()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        if(!$conn){
            phpcommon\sendError(ERR_USER_BASE + 1,'没有这个指挥官');
            return;
        }
        $commander_id = $_REQUEST['commander_id'];
        $row = $conn->execQueryOne('SELECT * FROM commander WHERE accountid=:accountid AND commander_id=:commander_id;',
                                   array(
                                       ':accountid' => $account_id,
                                       ':commander_id' => $commander_id,
                                   ));
        if (!$row) {
            phpcommon\sendError(ERR_USER_BASE + 1,'没有这个指挥官');
            return;
        }
        $c = $this->getCommander($commander_id);
        if(!$c){
            phpcommon\sendError(ERR_USER_BASE + 1,'没有这个指挥官');
            return;
        }
        $ex_level = $c['max_lv'];
        $p = $this->getParameter(SKIN_SKILL_TIME);
        $time = $p['param_value'];
        $ret = $conn->execScript('UPDATE commander SET active_time=:active_time, experience_level=:experience_level, modify_time=:modify_time ' .
                                 ' WHERE accountid = :account_id AND commander_id = :commander_id;',
                                 array(
                                     ':account_id' => $account_id,
                                     ':commander_id' => $commander_id,
                                     ':active_time' => time() + $time,
                                     ':experience_level' => $ex_level,
                                     ':modify_time' => time()
                                 ));
        if(!$ret){
            die();
            return;
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'commander_id' => $row['commander_id'],
            'time' => time() + $time,
            'experience_level' => $ex_level,
        ));
    }
}
?>
