<?php

require 'classes/Quest.php';
require 'classes/AddReward.php';

class QuestController{

    protected function getMysql($account_id)
    {
        $mysql_conf = getMysqlConfig(crc32($account_id));
        $conn = new phpcommon\Mysql(array(
            'host' => $mysql_conf['host'],
            'port' => $mysql_conf['port'],
            'user' => $mysql_conf['user'],
            'passwd' => $mysql_conf['passwd'],
            'dbname' => 'gamedb2002_' . $mysql_conf['instance_id']
        ));
        return $conn;
    }

    protected function getQuest($quest_id)
    {
        $quset_meta_table = require('../res/task@task.php');
        $quest_meta = getQuestConfig($quset_meta_table, $quest_id);
        $q = array(
            'id' => $quest_meta['id'],
            'type' => $quest_meta['type'],
            'condition' => $quest_meta['condition'],
            'value' => $quest_meta['value'],
            'active_value' => $quest_meta['active_value'],
            'reward' => $quest_meta['reward'],
        );
        return $q;
    }

    protected function getTaskReward($taskReward_id)
    {
        $taskReward_meta_table = require('../res/taskReward@taskReward.php');
        $taskReward_meta = getTaskRewardConfig($taskReward_meta_table, $taskReward_id);
        $t = array(
            'id' => $taskReward_meta['id'],
            'type' => $taskReward_meta['type'],
            'value' => $taskReward_meta['value'],
            'reward' => $taskReward_meta['reward']
        );
        return $t;
    }

    protected function getExplode($string)
    {
        $delim = "|";
        $drop_multiply = explode($delim, $string);
        $delim1 = ":";
        $arr = array();
        for ($i = 0; $i < count($drop_multiply); $i++) {
            $mul = explode($delim1, $drop_multiply[$i]);
            array_push($arr, $mul);
        }
        return $arr;
    }

    protected function getDrop($drop_id)
    {
        $drop_meta_table = require('../res/drop@drop.php');
        $drop_meta = getDropConfig($drop_meta_table, $drop_id);
        $d = array(
            'drop_id' => $drop_meta['drop_id'],
            'item_id' => $drop_meta['item_id'],
            'num' => $drop_meta['num'],
            'weight' => $drop_meta['weight'],
            'type' => $drop_meta['type']
        );
        return $d;
    }

    protected function insertActive($account_id, $active_id)
    {
        $conn = $this->getMysql($account_id);
        $ret = $conn->execScript('INSERT INTO quest(accountid, quest_id, quest_type, quest_state, create_time, modify_time) ' .
                                 ' VALUES(:accountid, :quest_id, :quest_type, :quest_state, :create_time, :modify_time) ' .
                                 ' ON DUPLICATE KEY UPDATE accountid=:accountid, quest_id=:quest_id, quest_type=:quest_type, quest_state=:quest_state, modify_time=:modify_time;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':quest_id' => $active_id,
                                     ':quest_type' => 0,
                                     ':quest_state'=> 0,
                                     ':create_time' => time(),
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
    }

    public function updateActive($account_id, $active_num)
    {
        $conn = $this->getMysql($account_id);
        $quest_id = 73001;
        $row = $conn->execQueryOne('SELECT * FROM quest WHERE accountid=:accountid AND quest_id=:quest_id;',
                                   array(
                                       ':accountid' => $account_id,
                                       ':quest_id' => $quest_id
                                   ));
        $t = $this->getTaskReward($quest_id);
        if ($row['quest_state'] == 0 && $active_num >= $t['value']) {
            $ret = $conn->execScript('UPDATE quest SET quest_state=:quest_state, modify_time=:modify_time ' .
                                     ' WHERE accountid=:accountid AND quest_id =:quest_id;',
                                     array(
                                         ':accountid' => $account_id,
                                         ':quest_id' => $quest_id,
                                         ':quest_state' => 1,
                                         ':modify_time' => time()
                                     ));
            if (!$ret) {
                die();
            }
        }
    }

    protected function updateActiveTime($account_id, $quest_id)
    {
        $conn = $this->getMysql($account_id);
        $ret = $conn->execScript('UPDATE active SET num_time=:num_time, active_num=:active_num, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':num_time' => time(),
                                     ':active_num' => 0,
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
        $ret = $conn->execScript('UPDATE quest SET quest_state=:quest_state, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid AND quest_type=:quest_type AND quest_id=:quest_id;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':quest_type' => 0,
                                     ':quest_id' => $quest_id,
                                     ':quest_state' => 0,
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
    }

    public function questInfo()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        $quest_list = array();
        $active_list = array();
        $achievement_list = array();
        $active_num = 0;
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $row = $conn->execQueryOne('SELECT active_num, num_time FROM active WHERE accountid=:accountid;',
                                   array(
                                       ':accountid' => $account_id,
                                   ));
        if ($row) {
            //每日更新活跃度
            if (phpcommon\getdayseconds(time()) - phpcommon\getdayseconds($row['num_time']) > 0) {
                $this->updateActiveTime($account_id, 73001);
            } else {
                $active_num = $row['active_num'];
            }
        }
        $rowCount = $conn->execQueryRowCount('SELECT * FROM quest WHERE accountid=:accountid;',
                                             array(
                                                 ':accountid' => $account_id,
                                             ));
        if ($rowCount != 0) {
            $rows = $conn->execQuery('SELECT * FROM quest WHERE accountid=:accountid AND quest_type=:quest_type;',
                                     array(
                                         ':accountid' => $account_id,
                                         ':quest_type' => 1
                                     ));
            if ($rows) {
                foreach ($rows as $row) {
                    array_push($quest_list, array(
                        'type' => $row['quest_type'],
                        'quest_id' => $row['quest_id'],
                        'quest_num' => $row['quest_num'],
                        'quest_state' => $row['quest_state']
                    ));
                }
            }
            $rows = $conn->execQuery('SELECT * FROM quest WHERE accountid=:accountid AND quest_type=:quest_type;',
                                     array(
                                         ':accountid' => $account_id,
                                         ':quest_type' => 2
                                     ));
            if ($rows) {
                foreach ($rows as $row) {
                    array_push($achievement_list, array(
                        'type' => $row['quest_type'],
                        'achievement_id' => $row['quest_id'],
                        'achievement_num' => $row['quest_num'],
                        'achievement_state' => $row['quest_state']
                    ));
                }
            }
            $rows = $conn->execQuery('SELECT * FROM quest WHERE accountid=:accountid AND quest_type=:quest_type;',
                                     array(
                                         ':accountid' => $account_id,
                                         ':quest_type' => 0
                                     ));
            if ($rows) {
                foreach ($rows as $row) {
                    array_push($active_list, array(
                        'type' => $row['quest_type'],
                        'active_id' => $row['quest_id'],
                        'active_state' => $row['quest_state']
                    ));
                }
            }
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'active_num' => $active_num,
            'active_list' => $active_list,
            'quest_list' => $quest_list,
            'achievement_list' => $achievement_list
        ));
    }

    public function submitQuest()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $quest_id = $_REQUEST['quest_id'];
        $quest_type = $_REQUEST['type'];
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $row = $conn->execQueryOne('SELECT * FROM quest WHERE accountid=:accountid AND quest_id=:quest_id AND quest_type=:quest_type;',
                                   array(
                                       ':accountid' => $account_id,
                                       ':quest_id' => $quest_id,
                                       ':quest_type' => $quest_type
                                   ));
        if (!$row) {
            phpcommon\sendError(ERR_USER_BASE + 2, '没有这个任务');
            return;
        }
        $item_list = array();
        if ($row['quest_state'] != 1) {
            phpcommon\sendError(ERR_USER_BASE + 3, '任务未完成');
            return;
        }
        $ret = $conn->execScript('UPDATE quest SET quest_state=:quest_state, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid AND quest_id =:quest_id AND quest_type=:quest_type;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':quest_id' => $quest_id,
                                     ':quest_type' => $quest_type,
                                     ':quest_state' => 2,
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
            return;
        }
        //发奖励
        if ($quest_type == 0) {
            $t = $this->getTaskReward($quest_id);
            if (!$t) {
                phpcommon\sendError(ERR_USER_BASE + 2, '没有这个任务');
                return;
            }
            $d = $this->getDrop($t['reward']);
            if (!$d) {
                phpcommon\sendError(ERR_USER_BASE + 3, '没有这个奖励');
                return;
            }
            $weight_sum = 0;
            $weight_array = $this->getExplode($d['weight']);
            for ($i = 0; $i < count($weight_array); $i++) {
                $weight_sum += $weight_array[$i][0];
            }

            $random = Rand(0, $weight_sum);
            $weight = 0;
            for ($i = 0; $i < count($weight_array); $i++) {
                $weight += $weight_array[$i][0];
                if ($weight > $random) {
                    $key = $i;
                    break;
                }
            }
            $item_id_array = $this->getExplode($d['item_id']);
            $num_array = $this->getExplode($d['num']);
            $item_id = $item_id_array[$key][0];
            $item_num = $num_array[$key][0];
            array_push($item_list, array(
                'item_id' => $item_id,
                'item_num' => $item_num,
            ));
            $addreward = new classes\AddReward();
            $addreward->addReward($item_id, $item_num, $account_id);
        } else {
            $q = $this->getQuest($quest_id);
            if (!$q) {
                phpcommon\sendError(ERR_USER_BASE + 2, '没有这个任务');
                return;
            }
            $array = $this->getExplode($q['reward']);
            $reward_id = $array[0][0];
            $reward_num = $array[0][1];
            $addreward = new classes\AddReward();
            $addreward->addReward($reward_id, $reward_num, $account_id);
            array_push($item_list, array(
                'item_id' => $reward_id,
                'item_num' => $reward_num,
            ));
        }
        if ($row['quest_type'] == 1) {
            $active = $conn->execQueryOne('SELECT active_num FROM active WHERE accountid=:accountid;',
                                          array(
                                              ':accountid' => $account_id
                                          ));
            //增加活跃度
            if (!$active) {
                $this->insertActive($account_id, 73001);
                $ret = $conn->execScript('INSERT INTO active(accountid, active_num, num_time, create_time, modify_time) ' .
                                         ' VALUES(:accountid, :active_num, :num_time, :create_time, :modify_time) ' .
                                         ' ON DUPLICATE KEY UPDATE accountid=:accountid, active_num=:active_num, num_time=:num_time, modify_time=:modify_time;',
                                         array(
                                             ':accountid' => $account_id,
                                             ':active_num' => $q['active_value'],
                                             ':num_time' => time(),
                                             ':create_time' => time(),
                                             ':modify_time' => time()
                                         ));
                if (!$ret) {
                    die();
                    return;
                }
                $this->updateActive($account_id, $q['active_value']);
            } else {
                $active['active_num'] = $q['active_value'] + $active['active_num'];
                $ret = $conn->execScript('UPDATE active SET active_num=:active_num, modify_time=:modify_time ' .
                                         ' WHERE accountid=:accountid;',
                                         array(
                                             ':accountid' => $account_id,
                                             ':active_num' => $active['active_num'],
                                             ':modify_time' => time()
                                         ));
                if (!$ret) {
                    die();
                    return;
                }
                $this->updateActive($account_id, $active['active_num']);
            }
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'item_list' => $item_list
        ));
    }

}
?>
